/*
** PGP5GUI - A GUI using Magic User Interface v3.8
**
** Copyright 23-JUNE-1998 by Stefan Zakarias, All Rights Reserved.
**
** This source code is released as FREEWARE - Use it for whatever you like,
** as long as NO financial reward is gained by you for such usage.
**
** If you use any parts of the this source code for anything, give ME credit
** wherever credit is due, please ;-)
*/

/*
** Functions to handle the interface between PGPS and the GUI
*/

/* Library stuff */
#include <libraries/mui.h>

/* Prototypes */
#ifdef __GNUC__
#include <clib/muimaster_protos.h>
#include <proto/alib.h>
#endif

#include <proto/muimaster.h>
#include <proto/exec.h>
#include <clib/alib_protos.h>
#include <clib/alib_stdio_protos.h>
#include <proto/dos.h>

/*  Ansi  */
#include <string.h>

/* Include generated by GenCodeC */
#include "PGP5GUI.h"

extern char *pgp5_tmp, *pgp5_tmp_asc;

extern char *armor_on_str, *armor_off_str;
extern char *txmode_on_str, *txmode_off_str;
extern char *sig_mode, *sig_options, *sig_files;
extern char *sigclippath, *sigclipfile, *sigclipfilename;
extern char *exec_buff;

extern BOOL GetSaveASLFileName(struct ObjApp *app, char *hail, char *pathbuff, char *filebuff, char *tofile);

extern void PGP5_Execute(UBYTE *command, struct ObjApp *app);
extern LONG GetCycle(Object *obj);

extern BOOL ReadClipToFile(char *tmp, struct ObjApp *app);
extern BOOL WriteFileToClip(char *tmp, struct ObjApp *app);

/*
** Do the Signing
*/
void
Do_Signing(struct ObjApp *App)
{
	char tofile[512];

	/* Title for the file requester */
	char title[] = "Save Signed Clip To (.asc) Text File...";

	char *sigfilesbuff = "\0";

	/* Values for the Cycle gadgets on the Signing page */
	LONG Sig_DETACH, Sig_USEARMOR, Sig_TEXTMODE, clipmode;

	clipmode = GetCycle(App->CY_Sign_CLIP);

	*sigclipfilename = '\0';

	switch (clipmode)
	{
		case 3:		/* Sign from clip to clip */
			if (ReadClipToFile(pgp5_tmp, App))
    	    {
				sigfilesbuff = pgp5_tmp;
			}

			break;

		case 2:		/* Sign from clip to file */
			if (ReadClipToFile(pgp5_tmp, App))
    	    {
				sigfilesbuff = pgp5_tmp;

	            /* Get name of file to save encrypted clip to... */
				if (GetSaveASLFileName(App, title, sigclippath, sigclipfile, tofile))
				{
					/* Build up the command line to send to DOS */
					sprintf(sigclipfilename, " +force -o \"%s\"", tofile);
				}
				else
				{
					/* Force exit */
					sigfilesbuff = "\0";
				}
			}

			break;

		case 1:		/* Sign from file to clip */
		    get(App->STR_PA_Sign_POPFILE,MUIA_String_Contents,&sigfilesbuff);

			/* If we have a file name, then do signing */
			if (*sigfilesbuff)
			{
				if (WriteFileToClip(sigfilesbuff, App))
            	{
					if (ReadClipToFile(pgp5_tmp, App))
    			    {
						sigfilesbuff = pgp5_tmp;
					}
					else
					{
						/* Force exit */
						sigfilesbuff = "\0";
					}
				}
				else
				{
					/* Force exit */
					sigfilesbuff = "\0";
				}
			}

			break;

    	default:	/* Sign from file to default (file) */
		    get(App->STR_PA_Sign_POPFILE,MUIA_String_Contents,&sigfilesbuff);
			break;
	}

	if (*sigfilesbuff)
	{
		strcpy(sig_files, sigfilesbuff);

		/* Using clipboard? */
		if (clipmode)
		{
			/* We lock these modes for clipboard use! */
			Sig_DETACH = FALSE;
			Sig_USEARMOR = TRUE;
			Sig_TEXTMODE = TRUE;
		}
		else
		{
			/* Get the values of the cycle gadgets */
			Sig_DETACH = GetCycle(App->CY_Sign_DETACH);
			Sig_USEARMOR = GetCycle(App->CY_Sign_MODE);
			Sig_TEXTMODE = GetCycle(App->CY_Sign_TEXTMODE);
		}

    	if (Sig_DETACH)
		{
			/* Make detached signature */
			strcpy(sig_mode,"-b");
		}
		else
		{
			/* Don't detach signature */
			*sig_mode = '\0';
		}

	    if (Sig_USEARMOR)
		{
			/* Use ASCII armor */
			strcpy(sig_options, armor_on_str);
		}
		else
		{
			/* Don't use ASCII armor */
			strcpy(sig_options, armor_off_str);
		}

		if (Sig_TEXTMODE)
		{
			/* Do universal compatible text mode */
			strcat(sig_options, txmode_on_str);
		}
		else
		{
			/* Don't do universal compatible text mode */
			strcat(sig_options, txmode_off_str);
		}

		/* Build up the command line to send to DOS */
		sprintf(exec_buff, "PGPS %s %s %s \"%s\"",
				sig_mode, sig_options, sigclipfilename, sig_files);

		/* Send the command to DOS */
		PGP5_Execute(exec_buff, App);

		if (clipmode)
		{
			/* clipmode not equal to 'clip to file'? */
			if (clipmode != 2)
			{
				/*
				** Signing from file/clip to clip...
				** Copy signed temporary file (T:pgp5.tmp.asc) to clip
				*/
				WriteFileToClip(pgp5_tmp_asc, App);
			}

			/*
			** Signing from file/clip to clip...
			** Delete signed temporary files (T:pgp5.tmp.asc, etc.)
			*/
			DeleteFile(pgp5_tmp_asc);
			DeleteFile(pgp5_tmp);
		}
	}
}
